Index: wp-includes/ms-functions.php
===================================================================
--- wp-includes/ms-functions.php	(revision 33486)
+++ wp-includes/ms-functions.php	(working copy)
@@ -1581,11 +1581,30 @@
 	 * @param string $password      User password.
 	 * @param array  $meta          Signup meta data.
 	 */
+
+	global $wpdb;
+
+	// Generate something random for a password reset key.
+	$key = wp_generate_password( 20, false );
+
+	do_action( 'retrieve_password_key', $user->user_login, $key );
+
+	// Now insert the key, hashed, into the DB.
+	if ( empty( $wp_hasher ) ) {
+		require_once ABSPATH . WPINC . '/class-phpass.php';
+		$wp_hasher = new PasswordHash( 8, true );
+	}
+	$hashed = time() . ':' . $wp_hasher->HashPassword( $key );
+	$wpdb->update( $wpdb->users, array( 'user_activation_key' => $hashed ), array( 'user_login' => $user->user_login ) );
+	
+	$resetlink = network_site_url( "wp-login.php?action=rp&key=$key&login=" . rawurlencode($user->user_login), 'login' );
+
 	$welcome_email = apply_filters( 'update_welcome_user_email', $welcome_email, $user_id, $password, $meta );
 	$welcome_email = str_replace( 'SITE_NAME', $current_site->site_name, $welcome_email );
 	$welcome_email = str_replace( 'USERNAME', $user->user_login, $welcome_email );
 	$welcome_email = str_replace( 'PASSWORD', $password, $welcome_email );
 	$welcome_email = str_replace( 'LOGINLINK', wp_login_url(), $welcome_email );
+	$welcome_email = str_replace( 'RESETLINK', $resetlink , $welcome_email );
 
 	$admin_email = get_site_option( 'admin_email' );
 
@@ -2136,16 +2155,17 @@
 	if ( !$text ) {
 		remove_filter( 'site_option_welcome_user_email', 'welcome_user_msg_filter' );
 
 		/* translators: Do not translate USERNAME, PASSWORD, LOGINLINK, SITE_NAME: those are placeholders. */
 		$text = __( 'Howdy USERNAME,
 
 Your new account is set up.
 
-You can log in with the following information:
 Username: USERNAME
-Password: PASSWORD
-LOGINLINK
 
+To set your password, visit the following address:
+
+<RESETLINK>
+
 Thanks!
 
 --The Team @ SITE_NAME' );
@@ -2136,16 +2155,17 @@
 		remove_filter( 'site_option_welcome_user_email', 'welcome_user_msg_filter' );
 
-		/* translators: Do not translate USERNAME, PASSWORD, LOGINLINK, SITE_NAME: those are placeholders. */
+		/* translators: Do not translate USERNAME, PASSWORD, LOGINLINK, RESETLINK, SITE_NAME: those are placeholders. */
 		$text = __( 'Howdy USERNAME,
 
